﻿using System;
using AspNetCoreSsrTemplateEngine.Template;

namespace AspNetCoreSsrTemplateEngine.DataBoundContext
{
    internal class StackedDataBindContext : IReadonlyDataBindContext
    {
        private readonly IReadonlyDataBindContext _underlying;
        private readonly IReadonlyDataBindContext _stackUp;

        public StackedDataBindContext(IReadonlyDataBindContext underlying, IReadonlyDataBindContext stackUp)
        {
            _underlying = underlying ?? throw new ArgumentNullException(nameof(underlying));
            _stackUp = stackUp ?? throw new ArgumentNullException(nameof(stackUp));
        }

        public bool TryGetValue(string key, out object value)
        {
            return _stackUp.TryGetValue(key, out value) || _underlying.TryGetValue(key, out value);
        }

        public bool TryGetTemplate(string key, out ITemplate slot)
        {
            return _stackUp.TryGetTemplate(key, out slot) || _underlying.TryGetTemplate(key, out slot);
        }
    }
}