﻿using AspNetCoreSsrTemplateEngine.DataBoundContext;
using AspNetCoreSsrTemplateEngine.Renders;
using System;
using System.Collections.Generic;
using System.IO;
using System.Threading.Tasks;

namespace AspNetCoreSsrTemplateEngine.Template
{
#if !EMBED_HTML

    public sealed class HotLoadTemplate : ITemplate, IDisposable
    {
        private readonly string _fullTypeName;
        private readonly string _fullPath;
        private readonly IComponentResolver _componentResolver;
        private readonly FileSystemWatcher _fileSystemWatcher;

        private IReadOnlyList<IRender> _renderSequence;

        public HotLoadTemplate(string fullTypeName, IComponentResolver componentResolver)
        {
            _fullTypeName = fullTypeName;
            _componentResolver = componentResolver;
            Reload();

            _fullPath = TemplateLoader.GetTemplateFilePath(fullTypeName);
            var dir = Path.GetDirectoryName(_fullPath);
            var fil = Path.GetFileName(_fullPath);
            _fileSystemWatcher = new FileSystemWatcher(dir, fil);

            _fileSystemWatcher.Renamed += _fileSystemWatcher_Changed;
            _fileSystemWatcher.Changed += _fileSystemWatcher_Changed;

            _fileSystemWatcher.EnableRaisingEvents = true;
        }

        private void _fileSystemWatcher_Changed(object sender, FileSystemEventArgs e)
        {
            if (e.FullPath == _fullPath)
            {
                Reload();
            }
        }

        private void Reload()
        {
            _renderSequence = TemplateLoader.LoadByFullTypeName(_fullTypeName, _componentResolver);
        }

        public async Task RenderAsync(IReadonlyDataBindContext dataBindContext, Stream stream)
        {
            foreach (var render in _renderSequence)
            {
                await render.RenderAsync(dataBindContext, stream);
            }
        }

        public void Dispose()
        {
            _fileSystemWatcher?.Dispose();
        }
    }

#endif
}
